@@ -30,6 +30,7 @@ gboolean config_output = FALSE, config_remember = TRUE, config_errorlist = TRUE,
 GTimeVal startime, pausedtime;
 gchar *grsync_dir, *rsync_help = NULL, *rsync_man = NULL;
 FILE *log_file = NULL;
+glong scroll_previous_run = 0;
 
 
 gboolean get_checkbox(void* main, gchar* name) {
@@ -77,6 +78,7 @@ void load_settings(GtkWidget* main, gchar* session, gchar *filename) {
 	if (session == NULL) {
 		session = g_key_file_get_start_group(settings_file);
 		save_settings(main, session, NULL);
+		session_last = -1;		// added because of save session bug, copied from external patch
 		load_groups(main, session);
 	}
 	set_main_title(session, NULL);
@@ -444,7 +446,7 @@ void on_play_clicked(GtkButton *button, gpointer user_data) {
 }
 
 
-void on_dryrun_clicked (GtkButton *button, gpointer user_data) {
+void on_dryrun_clicked(GtkButton *button, gpointer user_data) {
 	dryrunning = TRUE;
 	on_play_clicked(button, NULL);
 }
@@ -576,22 +578,27 @@ void set_global_progress(GtkProgressBar* progr, gdouble fraction) {
 }
 
 
-void scroll_to_end(GtkTextView* view) {
-	GtkTextBuffer *buffer;
-	GtkTextIter iter;
-	GtkTextMark *mark;
+void scroll_to_end(GtkTextView* view, gboolean final) {
+	GTimeVal curtime;
 
-	buffer = gtk_text_view_get_buffer(view);
-	gtk_text_buffer_get_end_iter(buffer, &iter);
-	mark = gtk_text_buffer_create_mark(buffer, "end_mark", &iter, TRUE);
-	gtk_text_view_scroll_mark_onscreen(view, mark);
-	gtk_text_buffer_delete_mark(buffer, mark);
+	if (!final) g_get_current_time(&curtime);
+	if (final || curtime.tv_sec - scroll_previous_run >= 1) {
+		GtkTextBuffer *buffer;
+		GtkTextIter iter;
+		GtkTextMark *mark;
+
+		scroll_previous_run = curtime.tv_sec;
+		buffer = gtk_text_view_get_buffer(view);
+		gtk_text_buffer_get_end_iter(buffer, &iter);
+		mark = gtk_text_buffer_create_mark(buffer, "end_mark", &iter, TRUE);
+		gtk_text_view_scroll_mark_onscreen(view, mark);
+		gtk_text_buffer_delete_mark(buffer, mark);
+	}
 }
 
 
 void rsync_cleanup(gpointer data) {
 	gchar tmps[50];
-	GTimeVal curtime;
 	glong elapsed;
 	GtkWidget *errors;
 	GtkTextBuffer *buffer;
@@ -615,11 +622,12 @@ void rsync_cleanup(gpointer data) {
 
 			buffer = gtk_text_view_get_buffer(view);
 			gtk_text_buffer_insert_at_cursor(buffer, comm_after, -1);
-			scroll_to_end(view);
+			scroll_to_end(view, FALSE);
 			if (config_log) fputs(comm_after, log_file);
 			g_free(comm_after);
 		}
 
+		scroll_to_end(view, TRUE);
 		gtk_button_set_label((GtkButton*) lookup_widget((GtkWidget*) data, "close"), "gtk-close");
 
 		GtkLabel *tmpl = (GtkLabel*) lookup_widget((GtkWidget*) data, "label_file");
@@ -680,7 +688,7 @@ gboolean out_watch(GIOChannel *source, GIOCondition condition, gpointer data) {
 		carriage = strchr(str->str, '\r');
 
 		gtk_text_buffer_insert_at_cursor(buffer, (gchar*)str->str, -1);
-		scroll_to_end((GtkTextView*)data);
+		scroll_to_end((GtkTextView*)data, FALSE);
 		if (config_log) fputs((gchar*)str->str, log_file);
 
 		if (!dryrunning) {
@@ -745,7 +753,7 @@ gboolean err_watch(GIOChannel *source, GIOCondition condition, gpointer data) {
 	//	if (status == G_IO_STATUS_NORMAL) {
 		gtk_text_buffer_get_end_iter(buffer, &iter);
 		gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, (gchar*)str->str, -1, "fore-red", NULL);
-		scroll_to_end((GtkTextView*)data);
+		scroll_to_end((GtkTextView*)data, FALSE);
 	//	}
 
 		if (config_log) {
